@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2005-2007  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2005-2008  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -30,12 +30,15 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
+#include "upa.h"
+
 #include "dialog.h"
 #include "tracer.h"
 #include "import.h"
 #include "export.h"
 
 static GtkWidget *notebook = NULL;
+static GtkWidget *menuitem_debug;
 static GtkWidget *menuitem_close;
 static GtkWidget *menuitem_print;
 static GtkWidget *menuitem_saveas;
@@ -279,6 +282,7 @@ static void close_callback(GtkWidget *item, gpointer user_data)
 	}
 
 	if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook)) == 0) {
+		gtk_widget_set_sensitive(menuitem_debug, FALSE);
 		gtk_widget_set_sensitive(menuitem_close, FALSE);
 		gtk_widget_set_sensitive(menuitem_print, FALSE);
 		gtk_widget_set_sensitive(menuitem_saveas, FALSE);
@@ -537,6 +541,7 @@ static gint open_file(const gchar *filename)
 		return -1;
 	}
 
+	gtk_widget_set_sensitive(menuitem_debug, TRUE);
 	gtk_widget_set_sensitive(menuitem_close, TRUE);
 	gtk_widget_show(toolbar_tracer);
 
@@ -581,6 +586,7 @@ static void recent_callback(GtkRecentChooser *chooser, gpointer user_data)
 
 	page = append_notebook_page(GTK_NOTEBOOK(notebook), child);
 
+	gtk_widget_set_sensitive(menuitem_debug, TRUE);
 	gtk_widget_set_sensitive(menuitem_close, TRUE);
 	gtk_widget_show(toolbar_tracer);
 
@@ -611,6 +617,7 @@ static void new_callback(GtkWidget *item, gpointer user_data)
 
 	page = append_notebook_page(GTK_NOTEBOOK(notebook), child);
 
+	gtk_widget_set_sensitive(menuitem_debug, TRUE);
 	gtk_widget_set_sensitive(menuitem_close, TRUE);
 	gtk_widget_show(toolbar_tracer);
 
@@ -662,6 +669,38 @@ static void fullscreen_callback(GtkWidget *item, gpointer user_data)
 		gtk_window_unfullscreen(GTK_WINDOW(window));
 }
 
+static gboolean print_packet(GtkTreeModel *model, GtkTreePath *path,
+					GtkTreeIter *iter, gpointer user_data)
+{
+	UpaPacket *packet;
+
+	gtk_tree_model_get(model, iter, COLUMN_UPA, &packet, -1);
+
+	if (packet != NULL) {
+		upa_packet_print(packet);
+		g_object_unref(packet);
+	}
+
+	return FALSE;
+}
+
+static void debug_callback(GtkWidget *item, gpointer user_data)
+{
+	GtkWidget *child;
+	GtkTreeModel *model;
+	gint page;
+
+	page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
+	if (page < 0)
+		return;
+
+	child = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), page);
+
+	model = g_object_get_data(G_OBJECT(child), "model");
+
+	gtk_tree_model_foreach(model, print_packet, NULL);
+}
+
 static void about_callback(GtkWidget *item, gpointer user_data)
 {
 	show_about_dialog();
@@ -801,7 +840,6 @@ static void create_window(void)
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(window), _("Bluetooth Analyzer"));
-	gtk_window_set_icon_name(GTK_WINDOW(window), "stock_bluetooth");
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_default_size(GTK_WINDOW(window), 850, 650);
 	g_signal_connect(G_OBJECT(window), "delete-event",
@@ -913,6 +951,20 @@ static void create_window(void)
 	gtk_widget_add_accelerator(item, "activate", accel,
 					GDK_F11, 0, GTK_ACCEL_VISIBLE);
 
+	item = gtk_menu_item_new_with_mnemonic(_("_Tools"));
+	gtk_container_add(GTK_CONTAINER(menubar), item);
+	menu = gtk_menu_new();
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
+
+	item = gtk_image_menu_item_new_with_mnemonic(_("_Debug Packet List"));
+	gtk_container_add(GTK_CONTAINER(menu), item);
+	g_signal_connect(item, "activate", G_CALLBACK(debug_callback), NULL);
+	gtk_widget_add_accelerator(item, "activate", accel,
+					GDK_F12, 0, GTK_ACCEL_VISIBLE);
+
+	menuitem_debug = item;
+	gtk_widget_set_sensitive(menuitem_debug, FALSE);
+
 	item = gtk_menu_item_new_with_mnemonic(_("_Help"));
 	gtk_container_add(GTK_CONTAINER(menubar), item);
 	menu = gtk_menu_new();
@@ -977,6 +1029,8 @@ int main(int argc, char *argv[])
 		gtk_exit(1);
 	}
 
+	gtk_window_set_default_icon_name("bluetooth");
+
 	create_window();
 
 	for (i = 1; i < argc; i++)
