@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2005-2007  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2005-2008  Marcel Holtmann <marcel@holtmann.org>
  *  Copyright (C) 2006-2007  Bastien Nocera <hadess@hadess.net>
  *
  *
@@ -46,6 +46,8 @@ static DBusGProxy *manager;
 
 static GtkWidget *notebook = NULL;
 
+static gboolean class_visibility = TRUE;
+
 static void update_tab_label(GtkWidget *child, const char *name)
 {
 	GtkWidget *label;
@@ -70,7 +72,9 @@ struct adapter_data {
 	GtkWidget *timeout_label;
 	GtkWidget *timeout_scale;
 	GtkWidget *entry;
-	GtkWidget *combo;
+	GtkWidget *class_box;
+	GtkWidget *class_label;
+	GtkWidget *class_combo;
 	GtkWidget *button_delete;
 	GtkWidget *button_trusted;
 	GtkWidget *button_disconnect;
@@ -227,6 +231,12 @@ static void class_callback(GtkWidget *combobox, gpointer user_data)
 	case 2:
 		minor = "laptop";
 		break;
+	case 3:
+		minor = "server";
+		break;
+	case 4:
+		minor = "handheld";
+		break;
 	default:
 		return;
 	}
@@ -360,38 +370,38 @@ static void type_to_icon(GtkTreeViewColumn *column, GtkCellRenderer *cell,
 {
 	guint type;
 
-	gtk_tree_model_get (model, iter, COLUMN_TYPE, &type, -1);
+	gtk_tree_model_get(model, iter, COLUMN_TYPE, &type, -1);
 
 	switch (type) {
 	case BLUETOOTH_TYPE_PHONE:
-		g_object_set (cell, "icon-name", "stock_cell-phone", NULL);
+		g_object_set(cell, "icon-name", "stock_cell-phone", NULL);
 		break;
 	case BLUETOOTH_TYPE_MODEM:
-		g_object_set (cell, "icon-name", "modem", NULL);
+		g_object_set(cell, "icon-name", "modem", NULL);
 		break;
 	case BLUETOOTH_TYPE_COMPUTER:
-		g_object_set (cell, "icon-name", "computer", NULL);
+		g_object_set(cell, "icon-name", "computer", NULL);
 		break;
 	case BLUETOOTH_TYPE_NETWORK:
-		g_object_set (cell, "icon-name", "network-wireless", NULL);
+		g_object_set(cell, "icon-name", "network-wireless", NULL);
 		break;
 	case BLUETOOTH_TYPE_HEADSET:
-		g_object_set (cell, "icon-name", "stock_headphones", NULL);
+		g_object_set(cell, "icon-name", "stock_headphones", NULL);
 		break;
 	case BLUETOOTH_TYPE_KEYBOARD:
-		g_object_set (cell, "icon-name", "input-keyboard", NULL);
+		g_object_set(cell, "icon-name", "input-keyboard", NULL);
 		break;
 	case BLUETOOTH_TYPE_MOUSE:
-		g_object_set (cell, "icon-name", "input-mouse", NULL);
+		g_object_set(cell, "icon-name", "input-mouse", NULL);
 		break;
 	case BLUETOOTH_TYPE_CAMERA:
-		g_object_set (cell, "icon-name", "camera-photo", NULL);
+		g_object_set(cell, "icon-name", "camera-photo", NULL);
 		break;
 	case BLUETOOTH_TYPE_PRINTER:
-		g_object_set (cell, "icon-name", "printer", NULL);
+		g_object_set(cell, "icon-name", "printer", NULL);
 		break;
 	default:
-		g_object_set (cell, "icon-name", "stock_bluetooth", NULL);
+		g_object_set(cell, "icon-name", "bluetooth", NULL);
 		break;
 	}
 }
@@ -491,8 +501,6 @@ GtkWidget *create_tree(GtkTreeModel *model, gboolean icons)
 				"Connected", gtk_cell_renderer_pixbuf_new(),
 					connected_to_icon, NULL, NULL);
 
-	column = gtk_tree_view_column_new();
-	gtk_tree_view_column_set_visible(column, FALSE);
 	gtk_tree_view_insert_column_with_data_func(GTK_TREE_VIEW(tree), -1,
 				"Trusted", gtk_cell_renderer_pixbuf_new(),
 					trusted_to_icon, NULL, NULL);
@@ -503,6 +511,8 @@ GtkWidget *create_tree(GtkTreeModel *model, gboolean icons)
 				"Bonded", gtk_cell_renderer_pixbuf_new(),
 					bonded_to_icon, NULL, NULL);
 
+	column = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_visible(column, FALSE);
 	gtk_tree_view_insert_column(GTK_TREE_VIEW(tree), column, -1);
 	gtk_tree_view_set_expander_column(GTK_TREE_VIEW(tree), column);
 
@@ -693,11 +703,16 @@ static void create_adapter(struct adapter_data *adapter)
 					G_CALLBACK(focus_callback), adapter);
 
 	vbox = gtk_vbox_new(FALSE, 6);
+	gtk_widget_set_no_show_all(vbox, TRUE);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
+	adapter->class_box = vbox;
+
 	label = create_label(_("Class of device"));
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
+	adapter->class_label = label;
+
 	combobox = gtk_combo_box_new_text();
 	gtk_combo_box_append_text(GTK_COMBO_BOX(combobox),
 						_("Unspecified"));
@@ -705,6 +720,10 @@ static void create_adapter(struct adapter_data *adapter)
 						_("Desktop workstation"));
 	gtk_combo_box_append_text(GTK_COMBO_BOX(combobox),
 						_("Laptop computer"));
+	gtk_combo_box_append_text(GTK_COMBO_BOX(combobox),
+						_("Server-class computer"));
+	gtk_combo_box_append_text(GTK_COMBO_BOX(combobox),
+						_("Handheld device"));
 	gtk_box_pack_start(GTK_BOX(vbox), combobox, FALSE, FALSE, 0);
 
 	if (major && minor && g_ascii_strcasecmp(major, "computer") == 0) {
@@ -714,6 +733,10 @@ static void create_adapter(struct adapter_data *adapter)
 			index = 1;
 		else if (g_ascii_strcasecmp(minor, "laptop") == 0)
 			index = 2;
+		else if (g_ascii_strcasecmp(minor, "server") == 0)
+			index = 3;
+		else if (g_ascii_strcasecmp(minor, "handheld") == 0)
+			index = 4;
 		else
 			index = -1;
 
@@ -721,7 +744,7 @@ static void create_adapter(struct adapter_data *adapter)
 	} else
 		gtk_widget_set_sensitive(GTK_WIDGET(combobox), FALSE);
 
-	adapter->combo = combobox;
+	adapter->class_combo = combobox;
 
 	g_signal_connect(G_OBJECT(combobox), "changed",
 					G_CALLBACK(class_callback), adapter);
@@ -792,6 +815,12 @@ static void create_adapter(struct adapter_data *adapter)
 	adapter->button_disconnect = button;
 
 	gtk_widget_show_all(mainbox);
+
+	if (class_visibility == TRUE) {
+		gtk_widget_show(adapter->class_combo);
+		gtk_widget_show(adapter->class_label);
+		gtk_widget_show(adapter->class_box);
+	}
 }
 
 static void mode_changed(DBusGProxy *object,
@@ -921,7 +950,8 @@ static void minor_changed(DBusGProxy *object,
 		else
 			index = -1;
 
-		gtk_combo_box_set_active(GTK_COMBO_BOX(adapter->combo), index);
+		gtk_combo_box_set_active(GTK_COMBO_BOX(adapter->class_combo),
+									index);
 	}
 }
 
@@ -1093,3 +1123,41 @@ void set_trusted(const char *adapter, const char *address)
 
 	g_object_unref(proxy);
 }
+
+static void class_hide(gpointer data, gpointer user_data)
+{
+	struct adapter_data *adapter = data;
+
+	if (adapter->attached == 0)
+		return;
+
+	gtk_widget_hide(adapter->class_box);
+	gtk_widget_hide(adapter->class_label);
+	gtk_widget_hide(adapter->class_combo);
+}
+
+void hide_class_widget(void)
+{
+	class_visibility = FALSE;
+
+	g_list_foreach(adapter_list, class_hide, NULL);
+}
+
+static void class_show(gpointer data, gpointer user_data)
+{
+	struct adapter_data *adapter = data;
+
+	if (adapter->attached == 0)
+		return;
+
+	gtk_widget_show(adapter->class_combo);
+	gtk_widget_show(adapter->class_label);
+	gtk_widget_show(adapter->class_box);
+}
+
+void show_class_widget(void)
+{
+	class_visibility = TRUE;
+
+	g_list_foreach(adapter_list, class_show, NULL);
+}
