@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2005-2007  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2005-2008  Marcel Holtmann <marcel@holtmann.org>
  *  Copyright (C) 2006-2007  Bastien Nocera <hadess@hadess.net>
  *
  *
@@ -26,3 +26,6 @@ GtkWidget *create_label(const gchar *str);
 GtkWidget *create_general(void);
 void setup_general(void);
 void cleanup_general(void);
+
+void hide_class_widget(void);
+void show_class_widget(void);
