@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2005-2007  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2005-2008  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -31,10 +31,10 @@
 
 #include <gtk/gtk.h>
 
-#include "bluetooth-device-selection.h"
-
 #include "marshal.h"
 
+#include "dialog.h"
+
 static DBusGConnection *conn = NULL;
 
 static GtkWidget *dialog;
@@ -42,73 +42,7 @@ static GtkWidget *label_filename;
 static GtkWidget *label_status;
 static GtkWidget *progress;
 
-static gint filesize = -1;
-
-static gchar *open_file_dialog(void)
-{
-	GtkWidget *dialog;
-	gchar *filename = NULL;
-
-	dialog = gtk_file_chooser_dialog_new(_("Select File"), NULL,
-				GTK_FILE_CHOOSER_ACTION_OPEN,
-				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
-
-	gtk_window_set_icon_name(GTK_WINDOW(dialog), "stock_bluetooth");
-
-	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
-		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
-
-	gtk_widget_destroy(dialog);
-
-	return filename;
-}
-
-static void selected_device_changed(BluetoothDeviceSelection *selector,
-					gchar *address, gpointer user_data)
-{
-	GtkWidget *dialog = user_data;
-
-	gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog),
-				GTK_RESPONSE_ACCEPT, address != NULL);
-}
-
-static gchar *browse_device_dialog(void)
-{
-	GtkWidget *dialog;
-	GtkWidget *selector;
-	gchar *address = NULL;
-
-	dialog = gtk_dialog_new_with_buttons(_("Select Device"),
-				NULL, GTK_DIALOG_NO_SEPARATOR,
-				GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
-				GTK_STOCK_CONNECT, GTK_RESPONSE_ACCEPT, NULL);
-
-	gtk_window_set_icon_name(GTK_WINDOW(dialog), "stock_bluetooth");
-
-	gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog),
-						GTK_RESPONSE_ACCEPT, FALSE);
-
-	gtk_window_set_default_size(GTK_WINDOW(dialog), 450, 300);
-
-	gtk_container_set_border_width(GTK_CONTAINER(dialog), 6);
-
-	selector = bluetooth_device_selection_new(_("Select destination device"));
-
-	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), selector);
-
-	g_signal_connect(selector, "selected-device-changed",
-				G_CALLBACK(selected_device_changed), dialog);
-
-	gtk_widget_show_all(selector);
-
-	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
-		g_object_get(selector, "device-selected", &address, NULL);
-
-	gtk_widget_destroy(dialog);
-
-	return address;
-}
+static guint64 filesize = 0;
 
 static void response_callback(GtkWidget *dialog,
 					gint response, gpointer user_data)
@@ -128,7 +62,6 @@ static void create_window(const gchar *filename)
 				GTK_DIALOG_NO_SEPARATOR,
 				GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
 
-	gtk_window_set_icon_name(GTK_WINDOW(dialog), "stock_bluetooth");
 	gtk_window_set_default_size(GTK_WINDOW(dialog), 400, -1);
 
 	gtk_container_set_border_width(GTK_CONTAINER(dialog), 6);
@@ -147,6 +80,7 @@ static void create_window(const gchar *filename)
 	gtk_box_pack_start(GTK_BOX(vbox), progress, FALSE, FALSE, 0);
 
 	label = gtk_label_new(NULL);
+	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 6);
 
@@ -168,25 +102,27 @@ static void create_window(const gchar *filename)
 }
 
 static void transfer_started(DBusGProxy *proxy, gchar *a, gchar *b,
-						gint size, gpointer user_data)
+					guint64 size, gpointer user_data)
 {
 	gchar *text;
 
 	filesize = size;
 
-	text = g_strdup_printf(_("Starting transfer of %d bytes"), size);
+	text = g_strdup_printf(_("Starting transfer of %jd bytes"), size);
 	gtk_label_set_markup(GTK_LABEL(label_status), text);
 	g_free(text);
 
 	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress), 0.0);
 }
 
-static void transfer_progress(DBusGProxy *proxy, guint bytes, gpointer user_data)
+static void transfer_progress(DBusGProxy *proxy,
+					guint64 bytes, gpointer user_data)
 {
 	gchar *text;
 	gdouble fraction;
 
-	text = g_strdup_printf(_("Transfered %d of %d bytes"), bytes, filesize);
+	text = g_strdup_printf(_("Transfered %jd of %jd bytes"),
+							bytes, filesize);
 	gtk_label_set_markup(GTK_LABEL(label_status), text);
 	g_free(text);
 
@@ -233,7 +169,8 @@ static void create_notify(DBusGProxy *proxy,
 	const gchar *path = NULL;
 
 	if (dbus_g_proxy_end_call(proxy, call, &error,
-			G_TYPE_STRING, &path, G_TYPE_INVALID) == FALSE) {
+					DBUS_TYPE_G_OBJECT_PATH, &path,
+						G_TYPE_INVALID) == FALSE) {
 		gchar *text, *message;
 
 		if (error != NULL) {
@@ -263,13 +200,13 @@ static void create_notify(DBusGProxy *proxy,
 				G_CALLBACK(session_connected), user_data, NULL);
 
 	dbus_g_proxy_add_signal(proxy, "TransferStarted", G_TYPE_STRING,
-				G_TYPE_STRING, G_TYPE_INT, G_TYPE_INVALID);
+				G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_INVALID);
 
 	dbus_g_proxy_connect_signal(proxy, "TransferStarted",
 				G_CALLBACK(transfer_started), NULL, NULL);
 
 	dbus_g_proxy_add_signal(proxy, "TransferProgress",
-						G_TYPE_UINT, G_TYPE_INVALID);
+						G_TYPE_UINT64, G_TYPE_INVALID);
 
 	dbus_g_proxy_connect_signal(proxy, "TransferProgress",
 				G_CALLBACK(transfer_progress), NULL, NULL);
@@ -283,6 +220,13 @@ static void create_notify(DBusGProxy *proxy,
 							G_TYPE_INVALID);
 }
 
+static void name_owner_changed(DBusGProxy *proxy, const char *name,
+			const char *prev, const char *new, gpointer user_data)
+{
+	if (g_str_equal(name, "org.openobex") == TRUE && *new == '\0')
+		gtk_main_quit();
+}
+
 static gchar *option_device = NULL;
 
 static GOptionEntry options[] = {
@@ -294,7 +238,8 @@ static GOptionEntry options[] = {
 int main(int argc, char *argv[])
 {
 	DBusGProxy *proxy;
-	GError *error;
+	GError *error = NULL;
+	GSList *filenames;
 	gchar *filename, *address;
 
 	bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
@@ -314,28 +259,24 @@ int main(int argc, char *argv[])
 		gtk_exit(1);
 	}
 
+	gtk_window_set_default_icon_name("bluetooth");
+
 	if (argc < 2) {
-		filename = open_file_dialog();
-		if (filename == NULL)
+		filenames = show_select_dialog();
+		if (filenames == NULL)
 			gtk_exit(1);
 	} else
-		filename = g_strdup(argv[1]);
+		filenames = g_slist_append(NULL, g_strdup(argv[1]));
 
 	if (option_device == NULL) {
-		address = browse_device_dialog();
+		address = show_browse_dialog();
 		if (address == NULL) {
-			g_free(filename);
+			g_slist_free(filenames);
 			gtk_exit(1);
 		}
 	} else
 		address = g_strdup(option_device);
 
-	dbus_g_object_register_marshaller(marshal_VOID__STRING_STRING_INT,
-				G_TYPE_NONE, G_TYPE_STRING,
-				G_TYPE_STRING, G_TYPE_INT, G_TYPE_INVALID);
-
-	error = NULL;
-
 	conn = dbus_g_bus_get(DBUS_BUS_SESSION, &error);
 	if (conn == NULL) {
 		if (error != NULL) {
@@ -348,22 +289,40 @@ int main(int argc, char *argv[])
 		gtk_exit(1);
 	}
 
+	dbus_g_object_register_marshaller(marshal_VOID__STRING_STRING_UINT64,
+				G_TYPE_NONE, G_TYPE_STRING,
+				G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_INVALID);
+
+	dbus_g_object_register_marshaller(marshal_VOID__UINT64,
+				G_TYPE_NONE, G_TYPE_UINT64, G_TYPE_INVALID);
+
+	filename = g_slist_nth_data(filenames, 0);
+
 	create_window(filename);
 
 	proxy = dbus_g_proxy_new_for_name(conn, "org.openobex",
 				"/org/openobex", "org.openobex.Manager");
 
+	dbus_g_proxy_add_signal(proxy, "NameOwnerChanged",
+		G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);
+
+	dbus_g_proxy_connect_signal(proxy, "NameOwnerChanged",
+				G_CALLBACK(name_owner_changed), NULL, NULL);
+
 	dbus_g_proxy_begin_call(proxy, "CreateBluetoothSession",
 				create_notify, filename, NULL,
 				G_TYPE_STRING, address, G_TYPE_STRING, "opp",
-				G_TYPE_BOOLEAN, FALSE, G_TYPE_INVALID);
+				G_TYPE_INVALID);
 
 	gtk_main();
 
+	g_object_unref(proxy);
+
 	dbus_g_connection_unref(conn);
 
 	g_free(address);
-	g_free(filename);
+
+	g_slist_free(filenames);
 
 	return 0;
 }
